import sinon from 'sinon';
export function stubObject(object, methods) {
    const stubObject = Object.assign({}, object);
    const objectMethods = getObjectMethods(object);
    const excludedMethods = [
        '__defineGetter__', '__defineSetter__', 'hasOwnProperty',
        '__lookupGetter__', '__lookupSetter__', 'propertyIsEnumerable',
        'toString', 'valueOf', '__proto__', 'toLocaleString', 'isPrototypeOf'
    ];
    for (const method in object) {
        if (typeof object[method] === 'function') {
            objectMethods.push(method);
        }
    }
    for (const method of objectMethods) {
        if (!excludedMethods.includes(method)) {
            // @ts-expect-error cannot index object by string
            stubObject[method] = object[method];
        }
    }
    if (Array.isArray(methods)) {
        for (const method of methods) {
            stubObject[method] = sinon.stub();
        }
    }
    else if (typeof methods === 'object') {
        for (const method in methods) {
            stubObject[method] = sinon.stub();
            stubObject[method].returns(methods[method]);
        }
    }
    else {
        for (const method of objectMethods) {
            // @ts-expect-error cannot index object by string
            if (typeof object[method] === 'function' && method !== 'constructor') {
                stubObject[method] = sinon.stub();
            }
        }
    }
    return stubObject;
}
export function stubConstructor(constructor, ...constructorArgs) {
    return stubObject(new constructor(...constructorArgs));
}
export function stubInterface(methods = {}) {
    const object = stubObject({}, methods);
    return new Proxy(object, {
        get: (target, name) => {
            if (!Object.prototype.hasOwnProperty.call(target, name) && name !== 'then') {
                target[name] = sinon.stub();
            }
            return target[name];
        }
    });
}
function getObjectMethods(object) {
    const methods = [];
    while (((object = Reflect.getPrototypeOf(object)) != null)) {
        const keys = Reflect.ownKeys(object);
        keys.forEach((key) => {
            if (typeof key === 'string') {
                methods.push(key);
            }
        });
    }
    return methods;
}
//# sourceMappingURL=index.js.map